#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{

    ui->setupUi(this);

    ui->textBrowser->setVisible(false);

    resize(0,0);

    ui->addCombo->setCurrentIndex(0);
    ui->debugLevelCombo->setCurrentIndex(1);

    /*QStringList searchpath;
    searchpath << "/bin" << "/usr/bin" << "/usr/local/bin" << QCoreApplication::applicationDirPath();
    QDir::setSearchPaths("bin", searchpath);*/

    QSettings settings("latex2rtf-gui");
    // Files and directories
    ui->texEdit->setText(settings.value("texFile").toString());
    ui->rtfEdit->setText(settings.value("rtfFile").toString());
    ui->bblEdit->setText(settings.value("bblFile").toString());
    ui->auxEdit->setText(settings.value("aucFile").toString());
    ui->configEdit->setText(settings.value("configDir").toString());
    // "use default" states
    ui->defRtfCheckBox->setCheckState((Qt::CheckState)settings.value("defRtf", 2).toInt());
    ui->defBblCheckBox->setCheckState((Qt::CheckState)settings.value("defBbl", 2).toInt());
    ui->defAuxCheckBox->setCheckState((Qt::CheckState)settings.value("defAux", 2).toInt());
    ui->defConfigCheckBox->setCheckState((Qt::CheckState)settings.value("defConfig", 2).toInt());
    // Equations and tables
    ui->displayedToRtfCheckBox->setCheckState((Qt::CheckState)settings.value("displayedToRtf", 2).toInt());
    ui->displayedToBitmapCheckBox->setCheckState((Qt::CheckState)settings.value("displayedToBitmap", 0).toInt());
    ui->inlineToRtfCheckBox->setCheckState((Qt::CheckState)settings.value("inlineToRtf", 2).toInt());
    ui->inlineToBitmapCheckBox->setCheckState((Qt::CheckState)settings.value("inlineToBitmap", 0).toInt());
    ui->equationsToTextCheckBox->setCheckState((Qt::CheckState)settings.value("equationsToText").toInt());
    ui->equationsToCommentsCheckBox->setCheckState((Qt::CheckState)settings.value("equationsToComments").toInt());
    ui->tablesToRtfCheckBox->setCheckState((Qt::CheckState)settings.value("tablesdToRtf", 2).toInt());
    ui->tablesToBitmapCheckBox->setCheckState((Qt::CheckState)settings.value("tablesToBitmap", 0).toInt());
    // Bitmaps
    ui->dpiSpinBox->setValue(settings.value("dpi", 300).toInt());
    ui->equationsScaleSpinBox->setValue(settings.value("equationsScale", 1.22).toDouble());
    ui->figuresScaleSpinBox->setValue(settings.value("figuresScale", 1.35).toDouble());
    ui->allFiguresToBitmapsCheckBox->setCheckState((Qt::CheckState)settings.value("allFiguresToBitmapsCheckBox", 0).toInt());
    // RTF
    ui->fieldsForEquationsCheckBox->setCheckState((Qt::CheckState)settings.value("fieldsForEquations", 2).toInt());
    ui->fieldsForRefCheckBox->setCheckState((Qt::CheckState)settings.value("fieldsForRef", 2).toInt());
    ui->escapeParenthesesCheckBox->setCheckState((Qt::CheckState)settings.value("escapeParentheses", 0).toInt());
    ui->semicolonsCheckBox->setCheckState((Qt::CheckState)settings.value("semicolons", 0).toInt());
    ui->addCombo->setCurrentIndex(settings.value("add", 0).toInt());
    // Language
    ui->languageCombo->addItem(settings.value("language", "-autodetect-").toString());
    ui->languageCombo->setCurrentIndex(0);
    // Codepage
    ui->codepageCombo->setCurrentIndex(0);
    // Debugging
    ui->debugLevelCombo->setCurrentIndex(settings.value("debugLevel", 1).toInt());
    ui->warningsCheckBox->setCheckState((Qt::CheckState)settings.value("warnings", 0).toInt());
    // Environment
    if(QFile::exists("/bin/latex") || QFile::exists("/usr/bin/latex") || QFile::exists("/usr/local/bin/latex"))
        ui->latexinstalledLabel->setText(tr("installed"));
    else
        ui->latexinstalledLabel->setText(tr("not installed (!)"));
    if(QFile::exists("/bin/convert") || QFile::exists("/usr/bin/convert") || QFile::exists("/usr/local/bin/convert"))
        ui->iminstalledLabel->setText(tr("installed"));
    else
        ui->iminstalledLabel->setText(tr("not installed (!)"));
    if(QFile::exists("/bin/gs") || QFile::exists("/usr/bin/gs") || QFile::exists("/usr/local/bin/gs"))
        ui->gsinstalledLabel->setText(tr("installed"));
    else
        ui->gsinstalledLabel->setText(tr("not installed (!)"));
    // LaTeX2RTF version (if installed)
    process = new QProcess(this);
    QString str;
    process->start("latex2rtf", QStringList("-v"));
    if (process->waitForStarted()){
        process->waitForFinished();
        str = process->readAllStandardOutput();
        QStringList list = str.split(" ");
        str = list[1];
    }
    else
        str = tr("not installed (!)");
    ui->latex2rtfVersionLabel->setText(str);

    // Window and tabs
    restoreGeometry(settings.value("windowGeometry").toByteArray());
    ui->tabWidget->setCurrentIndex(settings.value("currentTab").toInt());
    
    // Process initialization
    process->setProcessChannelMode(QProcess::MergedChannels); // merge stderr and stdout channels
    connect(process, SIGNAL(readyReadStandardOutput()), this, SLOT(UpdateTerminal()));
    connect(process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(AfterProcessFinished()));

    UpdateUI();
}

MainWindow::~MainWindow()
{
    if(process->state()) //running or starting (1 or 2)
        process->kill();

    QSettings settings("latex2rtf-gui");
    // Files and directories
    settings.setValue("texFile", ui->texEdit->text());
    settings.setValue("rtfFile", ui->rtfEdit->text());
    settings.setValue("bblFile", ui->bblEdit->text());
    settings.setValue("aucFile", ui->auxEdit->text());
    settings.setValue("configDir", ui->configEdit->text());
    // "use default" states
    settings.setValue("defRtf", ui->defRtfCheckBox->checkState());
    settings.setValue("defBbl", ui->defBblCheckBox->checkState());
    settings.setValue("defAux", ui->defAuxCheckBox->checkState());
    settings.setValue("defConfig", ui->defConfigCheckBox->checkState());
    // Equations and tables
    settings.setValue("displayedToRtf", ui->displayedToRtfCheckBox->checkState());
    settings.setValue("displayedToBitmap", ui->displayedToBitmapCheckBox->checkState());
    settings.setValue("inlineToBitmap", ui->inlineToBitmapCheckBox->checkState());
    settings.setValue("inlineToRtf", ui->inlineToRtfCheckBox->checkState());
    settings.setValue("equationsToText", ui->equationsToTextCheckBox->checkState());
    settings.setValue("equationsToComments", ui->equationsToCommentsCheckBox->checkState());
    settings.setValue("tablesdToRtf", ui->tablesToRtfCheckBox->checkState());
    settings.setValue("tablesToBitmap", ui->tablesToBitmapCheckBox->checkState());
    // Bitmaps
    settings.setValue("dpi", ui->dpiSpinBox->value());
    settings.setValue("equationsScale", ui->equationsScaleSpinBox->value());
    settings.setValue("figuresScale", ui->figuresScaleSpinBox->value());
    settings.setValue("allFiguresToBitmapsCheckBox", ui->allFiguresToBitmapsCheckBox->checkState());
    // RTF
    settings.setValue("fieldsForEquations", ui->fieldsForEquationsCheckBox->checkState());
    settings.setValue("fieldsForRef", ui->fieldsForRefCheckBox->checkState());
    settings.setValue("escapeParentheses", ui->escapeParenthesesCheckBox->checkState());
    settings.setValue("semicolons", ui->semicolonsCheckBox->checkState());
    settings.setValue("add", ui->addCombo->currentIndex());
    // Language
    settings.setValue("language", ui->languageCombo->currentText());
    settings.setValue("codepage", ui->codepageCombo->currentText());
    // Debugging
    settings.setValue("debugLevel", ui->debugLevelCombo->currentIndex());
    settings.setValue("warnings", ui->warningsCheckBox->checkState());
    // Window and tabs
    settings.setValue("currentTab", ui->tabWidget->currentIndex());
    settings.setValue("windowGeometry", saveGeometry());

    delete ui;
}


void MainWindow::on_texButton_clicked()
{
    QFileInfo fi(ui->texEdit->text());
    QString fileName = QFileDialog::getOpenFileName(this, tr("Input file (LaTeX)"), fi.path(), tr("TEX files (*.tex)"));

    if(fileName != QString())
        ui->texEdit->setText(fileName);
}


void MainWindow::on_rtfButton_clicked()
{
    QFileInfo fi(ui->texEdit->text());
    QString fileName = QFileDialog::getSaveFileName(this, tr("Output file (RTF)"), fi.path(), tr("RTF files (*.rtf)"));
    if(fileName != QString()){
        ui->defRtfCheckBox->setChecked(false);
        QFileInfo fi(fileName);
        ui->rtfEdit->setText(fi.completeBaseName()+".rtf");
    }
}

void MainWindow::on_bblButton_clicked()
{
    QFileInfo fi(ui->texEdit->text());
    QString fileName = QFileDialog::getOpenFileName(this, tr("BBL file"), fi.path(), tr("BBL files (*.bbl)"));
    if(fileName != QString()){
        ui->defBblCheckBox->setChecked(false);
        QFileInfo fi(fileName);
        ui->bblEdit->setText(fi.fileName());
    }
}


void MainWindow::on_auxButton_clicked()
{
    QFileInfo fi(ui->texEdit->text());
    QString fileName = QFileDialog::getOpenFileName(this, tr("AUX file"), fi.path(), tr("AUX files (*.aux)"));
    if(fileName != QString()){
        ui->defAuxCheckBox->setChecked(false);
        QFileInfo fi(fileName);
        ui->auxEdit->setText(fi.fileName());
    }
}

void MainWindow::on_configButton_clicked()
{
    QString dir = QFileDialog::getExistingDirectory(this, tr("Configuration directory"), ui->configEdit->text(), QFileDialog::ShowDirsOnly);
    if(dir != QString()){
        ui->defConfigCheckBox->setChecked(false);
        ui->configEdit->setText(dir);
    }
}

void MainWindow::on_helpButton_clicked()
{
    QDesktopServices::openUrl(QUrl("http://latex2rtf.sourceforge.net/latex2rtf_2_1_0.html", QUrl::TolerantMode));
}

void MainWindow::on_runButton_clicked()
{
    if(ui->textBrowser->isVisible()){
        ui->runButton->setText(tr("Run!"));
        ui->textBrowser->setVisible(false);
        ui->tabWidget->setVisible(true);
    }
    else{
        QFileInfo fi(ui->texEdit->text());
        if(!fi.exists())
            QMessageBox::warning(this, "LaTeX2RTF", tr("Specify a correct path to '.tex' file!"), QMessageBox::Ok);
        else{
            ui->tabWidget->setVisible(false);
            ui->textBrowser->setVisible(true);
            ui->runButton->setEnabled(false);
            ui->textBrowser->clear();
            ui->textBrowser->setTextInteractionFlags(Qt::NoTextInteraction);

            QStringList arguments;
            arguments << "latex2rtf";
            if(!ui->defConfigCheckBox->checkState())
                arguments << "-P \"" + ui->configEdit->text() + "\""; //configuration directory

            if(ui->codepageCombo->currentIndex() != 0)
                arguments << "-C " + ui->codepageCombo->currentText(); //codepage

            if(ui->debugLevelCombo->currentIndex() != 1)
                arguments << "-d" + ui->debugLevelCombo->currentText(); //debug level

            if(ui->dpiSpinBox->value() != 300)
                arguments << "-D" + ui->dpiSpinBox->text(); //bitmap resolution

            int i=0;
            if(ui->fieldsForEquationsCheckBox->checkState())
                i += 1;
            if(ui->fieldsForRefCheckBox->checkState())
                i += 2;
            if(i != 3)
                arguments << "-f" + QString::number(i); //which fields to use during conversion

            if(ui->allFiguresToBitmapsCheckBox->checkState())
                arguments << "-F"; //use LaTeX to create bitmaps for all figures

            if(ui->languageCombo->currentIndex() != 0)
                arguments << "-i " + ui->languageCombo->currentText(); //language

            i = 0;
            if(ui->displayedToRtfCheckBox->checkState())
                i += 1;
            if(ui->inlineToRtfCheckBox->checkState())
                i += 2;
            if(ui->displayedToBitmapCheckBox->checkState())
                i += 4;
            if(ui->inlineToBitmapCheckBox->checkState())
                i += 8;
            if(ui->equationsToCommentsCheckBox->checkState())
                i += 16;
            if(ui->equationsToTextCheckBox->checkState())
                i += 32;
            if(i != 3)
                arguments << "-M" + QString::number(i); //type of equation conversion

            if(ui->escapeParenthesesCheckBox->checkState())
                arguments << "-p"; //escape parentheses in mathematical formulas

            if(ui->equationsScaleSpinBox->value() != 1.22)
                arguments << "-se" + ui->equationsScaleSpinBox->text(); //scale for equation conversion

            if(ui->figuresScaleSpinBox->value() != 1.35)
                arguments << "-sf" + ui->figuresScaleSpinBox->text(); //scale for figure conversion

            i = 0;
            if(ui->tablesToRtfCheckBox->checkState())
                i += 1;
            if(ui->tablesToBitmapCheckBox->checkState())
                i += 2;
            if(i != 1)
                arguments << "-t" + QString::number(i); //type of table conversion

            if(ui->semicolonsCheckBox->checkState())
                arguments << "-S"; //semicolons should be used to separate arguments in RTF fields (instead of commas)

            if(ui->warningsCheckBox->checkState())
                arguments << "-W"; //includes warnings directly in the RTF file

            if(ui->addCombo->currentIndex() != 0)
                arguments << "-Z" + ui->addCombo->currentText(); //add the specified number of extra } to the end of the RTF file

            if(!ui->defAuxCheckBox->checkState())
                arguments << "-a\"" + ui->auxEdit->text() + "\""; //aux file

            if(!ui->defBblCheckBox->checkState())
                arguments << "-b\"" + ui->bblEdit->text() + "\""; //bbl file

            if(!ui->defRtfCheckBox->checkState())
                arguments << "-o\"" + ui->rtfEdit->text() + "\""; //output (rtf) file

            arguments << fi.fileName();
            ui->textBrowser->insertPlainText(arguments.join(" "));
            ui->textBrowser->insertPlainText("\n\n");

            process->setWorkingDirectory(fi.dir().absolutePath ());
            process->start(arguments.join(" "));
        }
    }
}

void MainWindow::UpdateUI()
{
    bool b;

    b = ui->defRtfCheckBox->isChecked();
    ui->rtfEdit->setEnabled(!b);
    if(b)
        ui->rtfEdit->setText((QFileInfo::QFileInfo(ui->texEdit->text())).completeBaseName()+".rtf");

    b = ui->defBblCheckBox->isChecked();
    ui->bblEdit->setEnabled(!b);
    if(b)
        ui->bblEdit->setText((QFileInfo::QFileInfo(ui->texEdit->text())).completeBaseName()+".bbl");

    b = ui->defAuxCheckBox->isChecked();
    ui->auxEdit->setEnabled(!b);
    if(b)
        ui->auxEdit->setText((QFileInfo::QFileInfo(ui->texEdit->text())).completeBaseName()+".aux");

    // Configuration directory
    b = ui->defConfigCheckBox->isChecked();
    ui->configEdit->setEnabled(!b);
    if(b){
        QFileInfo fi("bin:latex2rtf");
        QDir dir(fi.absolutePath());
        if(dir.cd("/share/latex2rtf/cfg") || dir.cd("/usr/share/latex2rtf/cfg") || dir.cd("/usr/local/share/latex2rtf/cfg"))
            ui->configEdit->setText(dir.absolutePath());
        else
            ui->configEdit->setText(tr("not found"));
    }

    // Language combo
    QString curlang = ui->languageCombo->currentText();
    QDir dir(ui->configEdit->text());
    QStringList filters("*.cfg");
    QFileInfoList filist = dir.entryInfoList(filters, QDir::Files, QDir::Name);
    ui->languageCombo->clear();
    ui->languageCombo->addItem(tr("-autodetect-"));
    for(int i=0; i<filist.size(); i++){
        QString basename = filist[i].baseName();
        if( basename!="direct" && basename!="fonts" && basename!="ignore" && basename!="style" )
            ui->languageCombo->addItem(basename);
    }
    ui->languageCombo->setCurrentIndex(0);
    ui->languageCombo->setCurrentIndex(ui->languageCombo->findText(curlang));
}

void MainWindow::UpdateTerminal()
{
    QString str = process->readAllStandardOutput();
    #ifdef Q_WS_WIN
        strlist = str.split("\r\n").join("\n"); //replaces '\r\n' by '\n'
    #endif
    ui->textBrowser->insertPlainText(str);
    ui->textBrowser->moveCursor(QTextCursor::End);
}

void MainWindow::AfterProcessFinished()
{
    ui->textBrowser->setTextInteractionFlags(Qt::TextBrowserInteraction);
    ui->runButton->setText(tr("OK"));
    ui->runButton->setEnabled(true);
    ui->textBrowser->insertPlainText(tr("Done! Click \"OK\" to continue..."));
    ui->textBrowser->moveCursor(QTextCursor::End);
}
